require 'crowd_fund/project'

module CrowdFund
  describe Project do

    before do
      @amount_to_add = 200
      @amount_to_remove = 20
      @project = Project.new("new_project", 100, 1000)
      $stdout = StringIO.new
    end

    it "has name in upcase form" do
      expect(@project.name).to eq("NEW_PROJECT")
    end

    it "has an initial target funding amount" do
      expect(@project.target).to eq(1000)
    end

    it "computes the total value of each received pledge" do
      @project.receive_pledge(Pledge.new(:bronze, 50))
      @project.receive_pledge(Pledge.new(:bronze, 50))
      @project.receive_pledge(Pledge.new(:gold, 100))
      @project.receive_pledge(Pledge.new(:silver, 75))
      @project.receive_pledge(Pledge.new(:silver, 75))

      yielded = []

      @project.each_pledge_received do |pledge|
        yielded << pledge
      end

      expect(yielded).to eq([
                             Pledge.new(:bronze, 100),
                             Pledge.new(:gold, 100),
                             Pledge.new(:silver, 150)
                            ])
    end

    it "computes a total of all received pledges" do
      expect(@project.total_pledges).to eq(0)

      @project.receive_pledge(Pledge.new(:bronze, 50))

      expect(@project.total_pledges).to eq(50)

      @project.receive_pledge(Pledge.new(:silver, 75))

      expect(@project.total_pledges).to eq(125)

      @project.receive_pledge(Pledge.new(:gold, 100))

      expect(@project.total_pledges).to eq(225)
    end

    it "has a string representation of the project" do
      expect(@project.to_s).to eq("\n#{@project.name} has an amount of #{@project.amount} towards a target of #{@project.target}.\nIt has #{@project.total_to_fund} to reach target (#{@project.stats}).")
    end

    it "computes the total funding outstanding as the target funding minus the funding amount" do
      expect(@project.total_to_fund).to eq(1000 - 100)
    end

    it "increases funds by an amount when funds are added" do
      @project.add(@amount_to_add)

      expect(@project.amount).to eq(300)
    end

    it "increases funds when pledges are received" do
      @project.receive_pledge(Pledge.new(:bronze, 50))

      expect(@project.amount).to eq(150)
    end

    it "decreases funds by an amount when funds are removed" do
      @project.remove(@amount_to_remove)

      expect(@project.amount).to eq(100 - @amount_to_remove)
    end

    context "has a default amount and target" do
      before do
        @default_project = Project.new("new_project")
      end
      it "has the default value of zero for funding amount" do
        expect(@default_project.amount).to eq(0)
      end

      it "has a default value of 1000 for target funding" do
        expect(@project.target).to eq(1000)
      end
    end

    context "has reached full funding" do
      before do
        @target = 1000
        @project = Project.new("new_project", @target, @target)
        $stdout = StringIO.new
      end
      it "is full" do
        @project.full?

        expect(@project).to be_full
      end

      it "has a full status" do
        expect(@project.stats).to eq("Full!")
      end
    end

    context "has not reached full funding" do
      before do
        @amount = 100
        @target = 1000
        @project = Project.new("new_project", @amount, @target)
        $stdout = StringIO.new
      end
      it "is full" do
        @project.full?

        expect(@project).not_to be_full
      end

      it "has a under funded status" do
        expect(@project.stats).to eq("Is under funded yet")
      end
    end

    context "in a collection of projects" do

      before do
        @project1 = Project.new("abc", 6000, 10000)
        @project2 = Project.new("lmn", 4000, 10000)
        @project3 = Project.new("xyz", 1000, 10000)

        @projects = [@project1, @project2, @project3]
      end

      it "sorts the list in decreased outstanding amount order" do
        expect(@projects.sort).to eq([@project3, @project2, @project1])
      end
    end
  end
end
